!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!                     CALCULATE DENSITY PROFILE    MARCH 2013                  |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine calculate_density_profile(params,density_profile,osolve,mat,        &
                                     initialize,istr,jstr,calc_comp_velo,      &
                                     write_compp, cloud_elem_mat_bins,          &
                                     cloud_elem_zpos_bins,cl)

use definitions

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! This routine calculates the density profile for a particular location on the |
! base of the model, using elemental material number to identify cells which   |
! are/are not compactible.  For compactible cells, an Athy curve is used to    |
! determine density profile with depth. 
!
! A temporary array of size matching the input density_str instance is used to |
! store the new density profile, at the same vertical devisions as the existing|
! profile.  In cases where the new density is greater than                     |
! density_profile(i)%density (the previous stored current density) or where the|
! new density is 0 (when erosion has occurred), density_profile(i)%density is  |
! replaced.                                      |
!
! This routine works from the top of the string downwards, storing total       |
! weight above a particular vertical location in the string, to inform Athy    |
! calculation.

!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

implicit none

include 'mpif.h'

type (parameters) params
type (string) density_profile

! How to specify a single instance of density_str, not the full structure?
! Note that we really only need density here, not densityp, so we could
! just past that, if easier?

! We also need material number for each element, and compaction parameters
! for compactible material types.

! We also need LSFs, to check if we're above/below the free surface.
! Suggest for now we consider only the free surface. This could be
! adapted to look for other LSFs, to better define transitions between
! material types.

type(octreesolve) osolve
type (material) mat(0:params%nmat)
integer :: initialize,istr,jstr, write_compp
logical :: calc_comp_velo(size(density_profile%density))

integer :: cloud_elem_mat_bins(osolve%nleaves,0:params%nmat)
double precision :: cloud_elem_zpos_bins(osolve%nleaves,0:params%nmat)
type (cloud) cl

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

!integer :: i,mat_maxi!,iproc,nproc,ierr
!logical :: mat_max_tie
integer :: iproc,nproc,ierr

type (string) density_temp

! How to specify that this is allocatable??
! We will need an allocatable array, density_temp, after the style of 
! density_profile (an instance of density_str)

double precision :: cur_lsf,x,y,z,z_free_surf,weight,eps
double precision :: dgrain,dfluid,spor,ccoef

double precision :: x0,y0,z0,dxyz, prev_zpos, zpos, topdown
double precision :: slice(0:params%nmat)
!stacking(0:params%nmat)
double precision :: interfaces(0:params%nmat)
integer :: matnum,i,j,k,free_surf_found,err,size_str
integer :: leaf,level,loc, tot_cl_particles, matnum_max, prev_elem_num
integer :: find_non_zero, found_tie


!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
! For the given input instance of density_str (locally stored as density_profile):
! 1. Allocate density_temp with dimensions after density_profile.
! 2. Starting with the top (end?) of density_temp, move down the string, at each
!    location:
!      a. Is this location above the free surface? Is so density_temp%density=0.
!         If not, continue.
!      b. Is the element cut?
!           - if so, use LSFs to determine what material number should be used
!             for the present location on the string.
!           - if not:
!               i. Query which element we're in.
!      			ii. Determine the element's material number.
!      c. Access compaction properties for this material number.
!      d. Determine density at this location.
!      e. Calculate weight contribution from this division of the string
!         (density*string division length*surface area for one string).
!           - add weight contribution to weight tally for this string.
!      f. Compare denity_profile%density(i) with density_temp%density(i),
!         overwrite density_profile%density(i) if:
!           - density_temp%density(i)>density_profile%density(i) OR
!           - density_temp%density(i)=0
! 3. Deallocate density_temp
!( Returns adjusted density_profile to compaction)

! Note: if using this routine to construct the original density profiles,
! could revise step f to always overprint density_profile, for all locations
! on the string.





call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)


eps = 1.d-10

size_str=size(density_profile%density)
x = 1.d0/(2.d0**dble(params%levelmax_oct))*(0.5d0+dble(istr-1))
y = 1.d0/(2.d0**dble(params%levelmax_oct))*(0.5d0+dble(jstr-1))

allocate (density_temp%density(size_str),stat=err)
density_temp%density=0.d0

! NOTE: as currently coded, this subroutine uses LSFs to determine material type.
!       In the future, this should be done only when initializing the density profile.
!       In subsequent calls, this subroutine should check the query the elemental
!       material number.

! NOTE: Once the option to not use LSFs to query material number has been implemented,
!       this routine should also check to see if the element corresponding to the current
!       location on the string is cut.  If so, use the LSF to determine material number,
!       if not, query the elemental material number.

free_surf_found=0
matnum=0
prev_elem_num=0

do i=size_str,1,-1
  ! Calculate current vertical position 
  ! Note we're now accounting for squishing here.
  z=(1.d0/(2.d0**dble(params%levelmax_oct))*params%vex)/dble(params%dstrnz)*(0.5d0+dble(i-1))

  ! If this call is initializing the density profiles, use LSFs to determine 
  ! material number


! 141114 J.Allen. If we are initializing compaction, use the LSFs to assign material 
! number. If not, continue with changes to assign material number based on cloud
! particle material numbers and positions, stored in cloud_elem_mat_bins and 
! cloud_elem_zpos_bins.

  if (initialize.gt.0) then

    ! Use LSFs to determine material type
    cur_lsf=1.d0
    do k=1,osolve%nlsf
      ! unsquished z into octree_interpolate
      call octree_interpolate (osolve%octree,osolve%noctree,osolve%icon,       &
                               osolve%nleaves,osolve%lsf(:,k),osolve%nnode,    &
                               x,y,z/params%vex,cur_lsf)
      
 !       if (iproc.eq.0) then
 !           write (*,*) 'density string output (',istr,',',jstr,')'
 !           write (*,*) 'x, y, z :',x,', ',y,', ',z
 !           write (*,*) 'level set function number (k): ',k
 !           write (*,*) 'cur_lsf ',cur_lsf
 !       endif
       
       
      if (cur_lsf < eps) then
        matnum = k
        
  !      if (iproc.eq.0) then
  !          write (*,*) 'cur_lsf ',cur_lsf,' is less than eps'
  !          write (*,*) 'matnum, k: ',matnum, k
  !      endif
       
        
        if (k==1 .and. free_surf_found==0) then
          z_free_surf=z
          free_surf_found=1
          
          
   !      if (iproc.eq.0) then
   !        if (jstr>0.48.and.jstr<0.62) then
   !           write (*,*) 'density string output (',istr,',',jstr,')'
   !           write (*,*) 'Free surface found at: ',z_free_surf,z
   !           write (*,*) 'level set function number (k): ',k
   !        endif
   !      endif
          
          
        endif
      endif
    enddo
  
  
  else


! 111114 J.Allen. Changes to make compaction not reliant on the LSFs
! Use LSFs to determine position of free surface, but then determine other interfaces
! between materials using cloud_elem_mat_bins and cloud_elem_zpos_bins



   ! Use LSFs to find position of the free surface
   ! For all points below the free surface, query octree_find_leaf to find element number,
   ! centre position of element, and edge dimension of element
   ! Note, this assigns matnum 1 to the first string position below the free surface, 
   ! which will not always be appropriate.  Would be better to assign this position's
   ! material number using the later part of the code.
   ! Would like to change this to increase i by one in the case where the free surface position is
   ! found; this will run the loop again for the first string position below the free 
   ! surface, and should assign this position's material number appropriately

    if (free_surf_found==0) then 
       cur_lsf=1.d0 
       k=1
       ! unsquished z into octree_interpolate
	   call octree_interpolate (osolve%octree,osolve%noctree,osolve%icon,       &
                               osolve%nleaves,osolve%lsf(:,k),osolve%nnode,    &
                               x,y,z/params%vex,cur_lsf)                     
       if (cur_lsf < eps) then
         matnum = k
         z_free_surf=z
         free_surf_found=1
 !        i=i+1
       endif
    else
!      call octree_find_leaf (osolve%octree,osolve%noctree,cl%x(i),cl%y(i),cl%z(i), &
!                        leaf,level,loc,x0,y0,z0,dxyz)   
      call octree_find_leaf (osolve%octree,osolve%noctree,x,y,z/params%vex, &
                        leaf,level,loc,x0,y0,z0,dxyz)   
      dxyz=dxyz*params%vex 
      z0=z0*params%vex
          
  !     if (iproc.eq.0) then
  !          write (*,*) 'x0,y0,z0:',x0,', ',y0,', ',z0
  !          write (*,*) 'dxyz:',dxyz
  !     endif
        
          
                  
       ! Assign material number based on information stored in cloud_elem_mat_bins
       ! (number of particles of each material number in each element) and 
       ! cloud_elem_zpos_bins (average vertical position of each material number
       ! in each element).
       
       ! compare element number against previous element number. Calculate
       ! interface positions only for the case where we've entered a new element

      if(prev_elem_num .ne. leaf) then
        tot_cl_particles=0
        slice=0
        interfaces=0
       
       
       
         !Determine number of cloud particles in this element
        do j=0,params%nmat
          tot_cl_particles=tot_cl_particles+cloud_elem_mat_bins(leaf,j)
        enddo
       
       
       
       !101214 debug output
  !     if (iproc.eq.0) then
  !          write (*,*) 'total cloud particles for this element:',tot_cl_particles
  !          write (*,*) 'dxyz:',dxyz
  !          write (*,*) 'slice thicknesses for this element, before:',slice(:)          
  !     endif
       
         !Assign a slice height for each material number 
         !'slice' is of length nmat+1, and stores the thickness for each material slice
  
        do j=0,params%nmat
        
  !        if (iproc.eq.0) then
  !           write (*,*) 'slice (j), before:',slice(j),' (',j,')'
  !           write (*,*) 'cloud_elem_mat_bins (leaf,j):',cloud_elem_mat_bins(leaf,j),' (',leaf,', ',j,')'
  !        endif
   
          slice(j)=dble(cloud_elem_mat_bins(leaf,j))/dble(tot_cl_particles)*dxyz
          
   !       if (iproc.eq.0) then
   !         write (*,*) 'slice (j), after:',slice(j),' (',j,')'
   !       endif
        enddo
        
        
        !101214 debug output
  !     if (iproc.eq.0) then
  !       if (jstr>0.48.and.jstr<0.62) then
  !          write (*,*) 'density string output (',istr,',',jstr,')'
  !          write (*,*) 'x0,y0,z0:',x0,', ',y0,', ',z0
  !          write (*,*) 'cloud_elem_mat_bins for this element (leaf):',cloud_elem_mat_bins(leaf,:),'(',leaf,')'
  !          write (*,*) 'slice thicknesses for this element, (leaf):',slice(:),'(',leaf,')'
  !       endif
  !     endif
        
        
       
         ! Determine the order of the slices
         ! 'stacking' is length nmat+1, and contains the ranking of position by material 
         !  number.  The material number with smallest ranking goes on top
         ! This is actually superfluous - could incorporate this into the section
         ! where material number is applied, and directly use the vertical positions, rather
         ! than the array 'stacking'
         !stacking=0
         !prev_zpos=maxval(cloud_elem_zpos_bins(leaf,:)+0.01
       
         !do j=0,params%nmat
         !  zpos=0
         !  matnum_max=0
         !  do k=0,params%nmat      
         !    if (cloud_elem_zpos_bins(leaf,k)>zpos .and. cloud_elem_zpos_bins(leaf,k)<prev_zpos) then
         !      zpos=cloud_elem_zpos_bins(leaf,k)
         !      matnum_max=k
         !    endif
         !  enddo
         !prev_zpos=zpos       
         !stacking(matnum_max)=j
         !enddo
     
     
       ! Define the interfaces between material types.
       ! Start with top most slice, and work to bottom most slice.  
       ! 'interfaces' stores the z position of the top of each material
       ! interfaces(1) stores the interface below material 0 and above material 1.
       ! For materials not found in this element, this should assign an interface at z=0
   
   
   
   
   
       ! 091214. Note that cloud_elem_zpos holds unsquished positions. Adjusted to
       ! account for squishing
       ! Also, this will have overlap of materials.  We do need the stacking part above,
       ! so that each point on the string is associated with only one material number
       ! After we have the stacking order correct, start at the top of the element and
       ! work down, assigning a position to the interfaces.
       ! interfaces stores the top of each material. Interfaces(1) stores the 
       ! position of the top of material 1.
       
        prev_zpos=maxval(cloud_elem_zpos_bins(leaf,:))+0.01
        topdown=0
        found_tie=0
        
      !101214 debug output
     !  if (iproc.eq.0) then
     !       write (*,*) 'previous zpos (max avg z posn of material types:',prev_zpos
     !  endif
        
        
        do j=0,params%nmat
           zpos=0
           matnum_max=0
           find_non_zero=0
!           prev_zeros=0
           do k=0,params%nmat      
             if (cloud_elem_zpos_bins(leaf,k)>zpos .and. cloud_elem_zpos_bins(leaf,k)<prev_zpos) then
                 find_non_zero=1      
                 zpos=cloud_elem_zpos_bins(leaf,k)
                 matnum_max=k
             elseif (cloud_elem_zpos_bins(leaf,k)>0.00001 .and. cloud_elem_zpos_bins(leaf,k).eq.zpos) then
                 if (found_tie.lt.1) then
                   found_tie=1
                   find_non_zero=1      
                   zpos=cloud_elem_zpos_bins(leaf,k)+.00001
                   matnum_max=k
                 endif
   
               
                !101214 debug output
      	!		 if (iproc.eq.0) then
        !		    write (*,*) 'zpos, for the highest avg z posn of material types (iteration j,k):',zpos,' (',j,', ',k,')'
        !		    write (*,*) 'matnum_max (iteration j,k):',matnum_max,' (',j,', ',k,')'
    	!	     endif
!             elseif (cloud_elem_zpos_bins(leaf,k)<eps.and.zpos<eps) then
!               zpos=cloud_elem_zpos_bins(leaf,k)
!               matnum_max=k
!                if (iproc.eq.0) then
!                    write (*,*) 'zero entry for cloud_elem_zpos_bins at j,k: ', j,', ',k
!        		    write (*,*) 'zpos, for the highest avg z posn of material types (iteration j,k):',zpos,' (',j,', ',k,')'
!        		    write (*,*) 'matnum_max (iteration j,k):',matnum_max,' (',j,', ',k,')'
!    		     endif
!               prev_zeros=1
             endif
           enddo
                  
           !stacking(matnum_max)=j
           !Assign interface position for matnum_max
         
           !interfaces(matnum_max)=z0+dxyz/2-topdown
            interfaces(matnum_max)=z0+dxyz-topdown
           
           if (find_non_zero.gt.0) then
             prev_zpos=zpos
!             if (found_tie.gt.0) then
!               prev_zpos=prev_zpos+.00001
!             endif
             topdown=topdown+slice(matnum_max)
           endif
           
      !     if (iproc.eq.0) then
      !             	write (*,*) 'prev_zpos (iteration):',prev_zpos,' (',j,')'
      !  		    write (*,*) 'matnum_max (iteration):',matnum_max,' (',j,')'
      !  		    write (*,*) 'slice(matnum_max) (iteration):',slice(matnum_max),' (',j,')'
      !  		    write (*,*) 'interface (iteration):',interfaces(matnum_max),' (',j,')'
      !  		    write (*,*) 'top of element (leaf):',z0+dxyz,' (',leaf,')'
      !  		    write (*,*) 'topdown (iteration):',topdown,' (',j,')'
      !		endif

           
         enddo
       
       
     !  if (iproc.eq.0) then
     !    if (jstr.eq.65) then
     !       write (*,*) 'density string output (',istr,',',jstr,')'
     !       write (*,*) 'x0,y0,z0:',x0,', ',y0,', ',z0
     !       write (*,*) 'cloud_elem_mat_bins for this element (leaf):',cloud_elem_mat_bins(leaf,:),'(',leaf,')'
     !       write (*,*) 'cloud_elem_zpos_bins for this element (leaf):',cloud_elem_zpos_bins(leaf,:),'(',leaf,')'
     !       write (*,*) 'slice thicknesses for this element, (leaf):',slice(:),'(',leaf,')'
     !       write (*,*) 'interfaces for this element, (leaf):',interfaces(:),'(',leaf,')'
     !    endif
     !  endif
       
       
       
       
      !      if (iproc.eq.0) then
      !             	write (*,*) 'interfaces, just defined (leaf):',interfaces(:),' (',leaf,')'

      !		endif
       
       
       ! do j=0,params%nmat
       !   interfaces(j)=cloud_elem_zpos_bins(leaf,j)*params%vex+slice(j)/2
       !enddo
       
      prev_elem_num=leaf
      endif
      
       ! Assign material number, comparing this string position to the interfaces in
       ! the current element
       ! For materials not found in this element, we exit at the first is statement,
       ! and do not check this string position against the interfaces.
       
       
       
      !      if (iproc.eq.0) then
      !             	write (*,*) 'interfaces, from previous (leaf):',interfaces(:),' (',leaf,')'
      !		endif
       
       
       ! Here we should be able to do something simpler, to compare against the interfaces
       ! and stacking arrays.
       
       ! 091214 Compare against interfaces, to assign material number.
       



! 291214: assign material number in order of highest to lowest surfaces

        prev_zpos=maxval(interfaces(:))+0.01
        
         do j=0,params%nmat
           zpos=z0
           matnum_max=0
           find_non_zero=0
           
       !     if (iproc.eq.0) then
       !       if (jstr.eq.65) then
       !            	write (*,*) 'zpos:',zpos
       !            	write (*,*) 'z:',z
       !            	write (*,*) 'interfaces (:):',interfaces(:)
       !       endif
       !	endif 
           
           
           do k=0,params%nmat      
             if (interfaces(k)>zpos .and. interfaces(k)<prev_zpos) then
               zpos=interfaces(k)
               matnum_max=k
               find_non_zero=1
      
               
   !           if (iproc.eq.0) then
   !             if (jstr.eq.65) then
   !                	write (*,*) 'j,k:',j,', ',k
   !                	write (*,*) 'matnum_max (k):',matnum_max,' (',k,')'
   !             endif
   !   		  endif 
      
      
             endif
           enddo
           
           if (find_non_zero.gt.0) then
             prev_zpos=zpos
             
    !          if (iproc.eq.0) then
    !            if (jstr.eq.65) then
    !               	write (*,*) 'z, j:',z,', ',j
    !               	write (*,*) 'matnum_max:',matnum_max
    !               	write (*,*) 'interfaces(matnum_max): ', interfaces(matnum_max)
    !               	write (*,*) 'prev_zpos:',prev_zpos,' (',k,')'
    !            endif
    ! 		  endif 
      		  
             
             if (z<interfaces(matnum_max)) then
               matnum=matnum_max
               
               
    !          if (iproc.eq.0) then
    !            if (jstr.eq.65) then
    !               	write (*,*) 'matnum, z: ', matnum,', ', z
    !            endif
    !  		  endif 
               
               
             endif
           endif
           
         enddo
                         




! older assignment of material number, based on lower to highest material number

  !     do j=0,params%nmat
            
      !      if (iproc.eq.0) then
      !             	write (*,*) 'checking nmat j, for (leaf):',j,' (',leaf,')'
      !             	write (*,*) 'z for mat j is (z):',j,' (',z,')'
      !             	write (*,*) 'interfaces(j):',interfaces(j),' (',j,')'
      !		endif 
    						
 !        if (z<interfaces(j)) then
 !          matnum=j
           
       !     if (iproc.eq.0) then
       !            	write (*,*) 'matnum j assigned to point (z):',j,' (',z,')'
       !		endif 
    	
    		
 !        endif
 !      enddo
       
       
       
       
       
          
       ! prev_zpos=maxval(cloud_elem_zpos_bins(leaf,:))+0.01
       
        !do j=0,params%nmat
        !  zpos=0
        !  matnum_max=0
        !  do k=0,params%nmat      
        !    if (cloud_elem_zpos_bins(leaf,k)>zpos .and. cloud_elem_zpos_bins(leaf,k)<prev_zpos) then
        !      zpos=cloud_elem_zpos_bins(leaf,k)
        !      matnum_max=k
        !    endif
        !  enddo
                 
          ! 091214. Here adjust because interfaces stores the base of the material, not 
          ! the top
                    
        !  if (z<interfaces(matnum_max)) then
        !    matnum =matnum_max
        !  endif      
        !  prev_zpos=zpos       
        !enddo
        
!      endif    
       
    endif


  endif

!    enddo       


  

    

  





    ! Update material numbers for material changes
    call mat_trans (params,mat,matnum,x,y,z)   
  !endif





  ! Assign density for this vertical location 
  if (mat(matnum)%compactible) then
    ! Calculate density according to Athy curve.
    ! Note:  for now this assumes that all compactible material is in a
    ! continuous stretch from the top of the string, and only one
    ! compactible material will be encountered.
      
    dgrain=mat(matnum)%grain_density
    dfluid=mat(matnum)%pore_fluid_density
    spor=mat(matnum)%surface_porosity
    ccoef=mat(matnum)%compaction_coef

    density_temp%density(i)=-(dgrain-(dgrain-dfluid)*spor*exp(-ccoef*(z_free_surf-z)))



! JANICE2: Commented out debug output below
!    if (jstr.eq.2**(params%levelmax_oct-1)) then
!      if (istr.eq.3 .or. istr.eq.2**(params%levelmax_oct-1) .or. istr.eq.2**(params%levelmax_oct)-3) then 
!
!
!        !if (iproc.eq.0) then
!          write (*,*) 'Compacting density'
!          write (*,*) 'density_temp%density(',i,'): ',density_temp%density(i)
!        !endif   
!
!
!      endif
!    endif





  else
    density_temp%density(i)=mat(matnum)%density



! JANICE2: Commented out debug output below
!    if (jstr.eq.2**(params%levelmax_oct-1)) then
!      if (istr.eq.3 .or. istr.eq.2**(params%levelmax_oct-1) .or. istr.eq.2**(params%levelmax_oct)-3) then 
!  
!  
!        !if (iproc.eq.0) then
!          write (*,*) 'Non-compacting density'
!          write (*,*) 'density_temp%density(',i,'): ',density_temp%density(i)
!        !endif  
!        
!          
!      endif
!    endif



  endif



 ! 100414 JCA 
    ! If this position of the string is current in a compactible material, and also
    ! this position was compactible at the last time step, set calc_comp_velo=T
      
    if (mat(matnum)%compactible.and.density_profile%compactiblep(i)) then
      !write (*,*) 'size(calc_comp_velo): ',size(calc_comp_velo)
      !write (*,*) 'i: ',i
      !write (*,*) 'size_str: ',size_str
      calc_comp_velo(i)=.true.
    endif
         
      ! 100414 JCA
      ! If this calculation of compaction is at the end of the timestep, overwrite
      ! compactiblep for this location with the current status (this will now store
      ! compactibility, for use in the next time step's calculations
    
    if (write_compp.gt.eps.or.initialize.gt.eps) then
      density_profile%compactiblep(i)=mat(matnum)%compactible
    endif

    
 
   !   if (jstr.eq.2**(params%levelmax_oct-1)) then
   !     if (istr.eq.3 .or. istr.eq.2**(params%levelmax_oct-1) .or. istr.eq.2**(params%levelmax_oct)-3) then 
   !       if (iproc.eq.0) then
   !         write (*,*) 'density string output (',istr,',',jstr,')'
   !         write (*,*) 'x, y, z :',x,', ',y,', ',z
   !         if (initialize.gt.0) then
   !           write (*,*) 'initial matnum :', matnum
   !           write (*,*) 'initial density :', density_temp%density(i)
   !         else
   !           write (*,*) 'matnum :', matnum
   !           write (*,*) 'density :', density_temp%density(i)
   !         endif
   !       endif    
   !     endif
   !   endif

    
    
    
    
  ! Increment weight above current position
  !weight=weight+density_temp(i)*(1/(2.d0**params%levelmax_oct))/params%dstrnz*(1/(2.d0**params%levelmax_oct))**2
enddo

! If initializing the density strings, copy all locations from the temporary
! array to to the current density profile.


if (initialize.gt.0) then


! JANICE2: Commented out debug output below
!	if (jstr.eq.2**(params%levelmax_oct-1)) then
!      if (istr.eq.3 .or. istr.eq.2**(params%levelmax_oct-1) .or. istr.eq.2**(params%levelmax_oct)-3) then 
!
!
!        !if (iproc.eq.0) then
!          write (*,*) 'Initializing density'
!        !endif  
!        
!          
!      endif
!    endif







  density_profile%density=density_temp%density



! JANICE2: Commented out debug output below
!	if (jstr.eq.2**(params%levelmax_oct-1)) then
!      if (istr.eq.3 .or. istr.eq.2**(params%levelmax_oct-1) .or. istr.eq.2**(params%levelmax_oct)-3) then 
!  
!  
!        !if (iproc.eq.0) then
!          write (*,*) 'density_profile%density: ',density_profile%density
!        !endif    
!        
!        
!      endif
!    endif






! If not initializing the density strings, check to see whether the density
! string should be updated at each location. Update only in cases where material
! has been eroded (density goes to 0), or where density at a given position on
! the string has increased.
else



! JANICE2: Commented out debug output below
!	if (jstr.eq.2**(params%levelmax_oct-1)) then
!      if (istr.eq.3 .or. istr.eq.2**(params%levelmax_oct-1) .or. istr.eq.2**(params%levelmax_oct)-3) then 
! 
! 
!        !if (iproc.eq.0) then
!          write (*,*) 'Updating density'
!        !endif    
!        
!        
!      endif
!    endif




  do i=1,size_str
    if (abs(density_temp%density(i)).gt.abs(density_profile%density(i))) then
      density_profile%density(i)=density_temp%density(i)

    ! JANICE: I changed the line below from the original version you coded
    ! Did you want to test the value was exactly equal to zero, because this
    ! should only be done as coded for integer values. I presume 
    !elseif (density_temp(i)==0) then
    elseif (abs(density_temp%density(i)) < eps) then
      density_profile%density(i)=density_temp%density(i)
    endif




! JANICE2: Commented out debug output below
!    if (jstr.eq.2**(params%levelmax_oct-1)) then
!      if (istr.eq.3 .or. istr.eq.2**(params%levelmax_oct-1) .or. istr.eq.2**(params%levelmax_oct)-3) then 
!
!
!        !if (iproc.eq.0) then
!          write (*,*) 'density_profile%density(',i,'): ',density_profile%density(i)
!        !endif   
!        
!         
!      endif
!    endif












  enddo
endif

deallocate (density_temp%density)

end subroutine calculate_density_profile

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------